# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")

letter_index = 0
def add_letter(ax = None, offset=0, offset2=0, letter=None):
    global letter_index

    if ax is None:
        ax = plt.gca()

    box = ax.get_position()
    x_off = (-0.5/2.54)/plt.gcf().get_figwidth()+offset
    if letter is None:
        letter = chr(ord("a") + letter_index)
        letter_index += 1
    plt.text(box.x0+x_off, box.y1+offset2, letter, fontproperties=get_letter_font_prop(), va="bottom", ha="center", transform=plt.gcf().transFigure, picker=True)

def get_letter_font_prop():
    from matplotlib.font_manager import FontProperties
    font = FontProperties()
    font.set_family("Helvetica Neue")
    font.set_weight("bold")
    font.set_size(letter_fontsize)
    return font

def add_letters(*args, **kwargs):
    for ax in plt.gcf().axes:
        add_letter(ax, *args, **kwargs)


def good_box(bp, color):
    # add styled boxed for a whisker box plot
    for box in bp['boxes']:
        box.set(color=color, linewidth=1)
    for w in bp['whiskers']:
        w.set(color=color, linewidth=1)
    for f in bp['fliers']:
        f.set(mec="gray", alpha=0.5, linewidth=1, ms=1)
    for m in bp['medians']:
        m.set(linewidth=2, color="C3")
    for c in bp['caps']:
        c.set(linewidth=1, color='k')


def hard_sigmoid(x, p1, p2, p3):
    return p1 * (x - p2) * (x >= p2) * (p1 * (x - p2) < p3) + p3 * (p1 * (x - p2) >= p3)


def hard_sigmoid_with_noiselevel(x, p1, p2, p3, const):
    return np.sqrt((hard_sigmoid(x, p1, p2, p3)) ** 2 + const ** 2)


def logistic_function(x):
    p3 = (60 - 25) / np.log(19)
    y = (0.01 / (1 + np.exp(-1 * ((x - 60) / p3))))
    return y


def logistic_function_with_noiselevel(x, const):
    y = np.sqrt((logistic_function(x) ** 2 + const ** 2))
    return y


# take every second line (to make the plot less crowded)
threshs_n = np.loadtxt('threshold_over_samplesize_2sigma_criterion.txt')
threshs_n = threshs_n[::2, :]
threshs_p = np.loadtxt('threshold_over_samplesize_5percent_criterion.txt')
threshs_p = threshs_p[::2, :]
trial_number = threshs_p[:, 0]

ax = plt.axes([0, 0, 0.1, 0.1], label="logistic")
ax_n = plt.axes([0.1, 0, 0.1, 0.1], label="2sigma")
ax_p = plt.axes([0.2, 0, 0.1, 0.1], label="5criterion")
ax_crit = plt.axes([0.3, 0, 0.1, 0.1], label="5criterion_different_p")


# Plot a, sigmoid curve with and without noise
width = 8

x = np.arange(-20, 150, 0.1)
y1 = logistic_function(x)
y2 = logistic_function_with_noiselevel(x, 0.003)

cmap2 = pylustrator.LabColormap(["c", "b"], 10)
cmap1 = pylustrator.LabColormap(["y", "g"], 10)

# plot the logistic function with noise
ax.plot(x, y2 * 1000, color=cmap1(5), label="$f(x)$")
# plot the logistic function without noise
ax.plot(x, y1 * 1000, color=cmap1(10), label="$f_0(x)$")

# plot a line for the threshold
ax.plot([25, 25], [0, 4], 'k--', lw=0.5)
# plot the noise level line
ax.plot([-20, 150], [3, 3], 'k--', lw=0.5)
# plot the saturation level line (without noise)
ax.plot([-20, 150], [10, 10], 'k--', lw=0.5)
# plot the saturation level line (with noise)
ax.plot([80, 150], [np.sqrt(10 ** 2 + 3 ** 2), np.sqrt(10 ** 2 + 3 ** 2)], 'k--', lw=0.5)

# set the axes of the plot
ax.set_xlabel('stimulus intensity (dB)')
ax.set_xlim(0, 150)
ax.set_ylabel('response amplitude')
ax.set_ylim(-1, 12)
ax.set_yticks([0, 0.003 * 1000, 0.01 * 1000])
ax.set_yticklabels(['0', '$\sigma$', '$a$'])
ax.set_ylim(bottom=0)
ax.text(-5, np.sqrt(3 ** 2 + 10 ** 2), '$\sqrt{a^2+\sigma^2}$')

# add a legend
ax.legend()


# Plot b, threshold of 2sigma criterion for different sample sizes
bp3 = ax_n.boxplot(np.transpose(threshs_n[:, 1::]), positions=trial_number, whis=[5, 95], widths=width)
good_box(bp3, cmap1(5))

# label the axes
ax_n.set_ylabel('threshold (dB)')
ax_n.set_xlabel('sample size')

# add a title
ax_n.text(400, 70, '2$\sigma$ criterion', color='gray', fontsize=30)

# set the ticks
ax_n.set_xticks(np.arange(200, 1000, 100))
ax_n.set_xticklabels(np.arange(200, 1000, 100))


# Plot c, threshold of 5% criterion for different sample sizes
bp2 = ax_p.boxplot(np.transpose(threshs_p[:, 1::]), positions=trial_number, whis=[5, 95], widths=width)
good_box(bp2, cmap1(5))

# label the axes
ax_p.set_ylabel('threshold (dB)')
ax_p.set_xlabel('sample size')

# add a title
ax_p.text(400, 35, '5% criterion', color='gray', fontsize=30)

# set the ticks
ax_p.set_xticks(np.arange(200, 1000, 100))
ax_p.set_xticklabels(np.arange(200, 1000, 100))
ax_p.set_ylim(15, 40)

add_letters()


# Plot d, threshold for different p values

def find_threshold(p):
    # calculate the threshold value for different p values
    popt = [10, 60, 11.89]
    threshold = -popt[2] * np.log(1 / p - 1) + popt[1]
    return threshold

# p values
p = np.arange(0, 1, 0.01)

# plot the p values and the corresponding threshold
ax_crit.plot(p, find_threshold(p), color=cmap1(5))

ax_crit.set_xlabel('criterion', labelpad=-8, fontdict=dict(size=8))
ax_crit.set_yticks([])
for tick in ax_crit.xaxis.get_major_ticks():
    tick.label.set_fontsize(8)
ax_crit.set_ylabel('threshold (dB)', fontdict=dict(size=8))
ax_crit.set_ylim(bottom=0)

# a line for a p value of 0.01
ax_crit.plot([0.01, 0.01], [0, find_threshold(0.01)], '--k', lw=0.5)
ax_crit.plot([0.01, 1], [find_threshold(0.01), find_threshold(0.01)], '--k', lw=0.5)
ax_crit.text(0.5, find_threshold(0.01), '1%')

# a line for a p value of 0.05
ax_crit.plot([0.05, 0.05], [0, find_threshold(0.05)], '--k', lw=0.5)
ax_crit.plot([0.05, 1], [find_threshold(0.05), find_threshold(0.05)], '--k', lw=0.5)
ax_crit.text(0.5, find_threshold(0.05), '5%')

# a line for a p value of 0.1
ax_crit.plot([0.1, 0.1], [0, find_threshold(0.1)], '--k', lw=0.5)
ax_crit.plot([0.1, 1], [find_threshold(0.1), find_threshold(0.1)], '--k', lw=0.5)
ax_crit.text(0.5, find_threshold(0.1), '10%')


# Plot e, plot of the hard sigmoid function
ax = plt.axes([0, 0.1, 0.1, 0.1], label="hard_sigmoid")

# the data
x = np.arange(0, 100, 0.1)
y_noise = hard_sigmoid_with_noiselevel(x, 0.003, 30, 0.1, 0.03)
y = hard_sigmoid(x, 0.003, 30, 0.1)

# lines for illustration
ax.plot([0, 100], [0, 0], 'k--', lw=0.5)
ax.plot([55, 100], [0.1, 0.1], 'k--', lw=0.5)
ax.plot([45, 100], [y_noise[-1], y_noise[-1]], 'k--', lw=0.5)
ax.plot([0, 50], [0.03, 0.03], 'k--', lw=0.5)
ax.plot([30, 30], [0.0, 0.04], 'k--', lw=0.5)

# the hard sigmoid without noise
ax.plot(x, y_noise, color=cmap2(5), label='$f(x)$')
ax.plot(x, y, color=cmap2(10), label='$f_0(x)$')

# lines to illustrate the slope of the curve
x1 = 47
x2 = 55
y = lambda x: hard_sigmoid(x, 0.003, 30, 0.1)
ax.plot([x1, x2], [y(x1), y(x1)], "k--", lw=0.5)
ax.plot([x2, x2], [y(x1), y(x2)], "k--", lw=0.5)
ax.legend(loc=0)

# limits and labels
ax.set_xlim(0, 100)
ax.set_ylim(-0.01, 0.11)
ax.set_xlabel('stimulus intensity (dB)')
ax.set_ylabel('response amplitude')
plt.ylim(0, 0.11)
plt.yticks([0, 0.03, 0.1], ["0", "$\sigma$", "h"])


# Plot f, plot of the threshold for the hard sigmoid function
threshs = np.loadtxt('threshold_over_samplesize_knee_criterion.txt')
threshs = threshs[::2, :]
trial_number = threshs[:, 0]

# add the axes
ax0 = plt.axes([0, 0.2, 0.1, 0.1], label="hard_sigmoid_threshold")

# plot the boxes
width = 8
bp2 = ax0.boxplot(np.transpose(threshs[::, 1::]), positions=trial_number, whis=[5, 95], widths=width)
good_box(bp2, cmap2(5))

# labels
ax0.set_xlabel('sample size')
ax0.set_xticks(np.arange(200, 1000, 100))
ax0.set_xticklabels(np.arange(200, 1000, 100))
ax0.set_ylabel('threshold (dB)')
ax0.set_ylim(20, 40)

add_letter(plt.gcf().axes[-2])
add_letter(plt.gcf().axes[-1])

#% start: automatic generated code from pylustrator
fig = plt.figure(1)
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.axes[2].texts[0].set_fontsize(10)
fig.axes[1].texts[0].set_fontsize(10)
fig.axes[0].texts[0].set_position([45.094838, 10.498769])  # Text: "$\sqrt{a^2+\sigma^2}$"
fig.axes[0].text(0.5, 0.5, 'New Text', transform=fig.axes[0].transAxes, rotation=90)
fig.axes[0].texts[1].set_text("threshold")
fig.axes[0].texts[1].set_position([0.143469, 0.696292])  # Text: "threshold"
fig.axes[0].get_legend()._set_loc((0.7467233731311799, 0.332586130296054))
fig.axes[3].texts[6].set_ha("right")
fig.axes[3].texts[5].set_ha("right")
fig.axes[3].texts[4].set_ha("right")
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes, rotation=90)
fig.axes[4].texts[0].set_text("threshold")
fig.axes[4].texts[0].set_ha("center")
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes)
fig.axes[4].texts[1].set_text(r"$\sigma$")
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes)
fig.axes[4].texts[2].set_text("h")
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes)
fig.axes[4].texts[3].set_text("$\sqrt{h^2 + \sigma^2}$")
fig.axes[4].texts[0].set_position([0.305483, 0.707097])  # Text: "threshold"
fig.axes[4].texts[2].set_position([0.503119, 0.849923])  # Text: "h"
fig.axes[4].texts[3].set_position([0.210488, 0.935618])  # Text: "$\sqrt{h^2 + \sigma^2}$"
fig.axes[4].get_legend()._set_loc((0.8004131019120349, 0.4141061920897698))
fig.axes[4].texts[1].set_position([0.527142, 0.235773])  # Text: "$\sigma$"
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes)
fig.axes[4].texts[4].set_text("s")
fig.axes[4].texts[4].set_position([0.567854, 0.435728])  # Text: "s"
fig.set_size_inches(15.980000/2.54, 13.590000/2.54, forward=True)
fig.axes[4].set_position([0.075580, 0.098928, 0.350254, 0.260294])
fig.axes[3].texts[2].set_position([0.525520, 0.693898])  # Text: "c"
fig.axes[5].texts[0].set_position([0.034431, 0.374369])  # Text: "e"
fig.axes[5].text(0.5, 0.5, 'New Text', transform=fig.axes[5].transAxes)
fig.axes[5].texts[2].set_text("knee criterion")
fig.axes[5].texts[2].set_color("#808080")
fig.axes[5].texts[2].set_position([1.004465, 0.287351])  # Text: "knee criterion"
fig.axes[5].texts[2].set_ha("right")
fig.axes[2].texts[0].set_ha("right")
fig.axes[1].texts[0].set_ha("right")
fig.axes[1].texts[0].set_position([901.852426, 51.842409])  # Text: "2$\sigma$ criterion"
fig.axes[3].annotate('New Annotation', (-0.0395, 0.0), (0.505, 60.04978625995511), arrowprops=dict(arrowstyle='->', linewidth=0.5))
fig.axes[3].annotate('New Annotation', (-0.0395, 0.0), (0.505, 60.04978625995511), arrowprops=dict(arrowstyle='->', linewidth=0.5))
fig.axes[3].set_position([0.857594, 0.591918, 0.108705, 0.098604])
fig.axes[3].texts[6].set_fontsize(8)
fig.axes[3].texts[6].set_position([2.633664, 352.504487])  # Text: "10%"
fig.axes[3].texts[5].set_fontsize(8)
fig.axes[3].texts[5].set_position([2.633664, 277.375577])  # Text: "5%"
fig.axes[3].texts[4].set_fontsize(8)
fig.axes[3].texts[4].set_position([1.343595, 5.551622])  # Text: "1%"
fig.axes[3].annotate('New Annotation', (-0.0395, 0.0), (0.505, 60.04978625995511), arrowprops=dict(arrowstyle='->', linewidth=0.5))
fig.axes[3].texts[7].set_text("10%")
fig.axes[3].texts[7].set_fontsize(8)
fig.axes[3].texts[7].set_position([1.343595, 81.225587])  # Text: "10%"
fig.axes[3].texts[7].set_ha("right")
fig.axes[3].texts[7].xy = (0.549790, 36.426138)
fig.axes[3].annotate('New Annotation', (-0.0395, 0.0), (0.505, 60.04978625995511), arrowprops=dict(arrowstyle='->', linewidth=0.5))
fig.axes[3].texts[8].set_text("5%")
fig.axes[3].texts[8].set_color("#5b5b5b")
fig.axes[3].texts[7].set_color("#808080")
fig.axes[3].texts[8].set_position([1.327669, 50.943252])  # Text: "5%"
fig.axes[3].texts[8].set_ha("right")
fig.axes[3].texts[8].set_fontsize(8)
fig.axes[3].texts[8].xy = (0.740912, 25.042970)
fig.axes[3].texts[4].set_color("#808080")
fig.axes[5].set_position([0.580923, 0.114975, 0.350254, 0.219747])
fig.axes[5].texts[1].set_position([0.525520, 0.342464])  # Text: "f"
fig.axes[3].texts[3].set_position([0.808509, 0.658606])  # Text: "d"
fig.axes[2].texts[0].set_position([901.856175, 18.649503])  # Text: "5% criterion"
fig.axes[3].texts[1].set_position([0.525520, 0.973621])  # Text: "b"
fig.axes[0].set_position([0.075580, 0.612479, 0.350254, 0.260294])
fig.axes[3].texts[0].set_position([0.034431, 0.881331])  # Text: "a"
fig.axes[3].texts[10].set_position([3.419599, 245.794182])  # Text: "New Annotation"
fig.axes[3].texts[10].xy = (3.082147, 68.299009)
fig.axes[3].texts[9].xy = (4.276655, 113.831681)
fig.axes[2].set_position([0.580923, 0.448154, 0.350254, 0.219747])
fig.axes[1].set_position([0.580923, 0.762641, 0.351225, 0.219747])
fig.axes[0].spines['right'].set_visible(False)
fig.axes[0].spines['top'].set_visible(False)
fig.axes[1].spines['right'].set_visible(False)
fig.axes[1].spines['top'].set_visible(False)
fig.axes[2].spines['right'].set_visible(False)
fig.axes[2].spines['top'].set_visible(False)
fig.axes[3].spines['right'].set_visible(False)
fig.axes[3].spines['top'].set_visible(False)
fig.axes[4].spines['right'].set_visible(False)
fig.axes[4].spines['top'].set_visible(False)
fig.axes[5].spines['right'].set_visible(False)
fig.axes[5].spines['top'].set_visible(False)
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3]+".pdf", dpi=300)
plt.savefig(__file__[:-3]+".png", dpi=300)

plt.show()
