# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import pylustrator

font = 10
letter_fontsize = font

mpl.rc('xtick', labelsize=font)
mpl.rc('ytick', labelsize=font)
mpl.rc('legend', fontsize=font)
mpl.rc('axes', labelsize=font, titlesize=font)
mpl.rc('mathtext', default="regular")
mpl.rc('figure', dpi=100)
mpl.rc('legend', frameon=False)
mpl.rc('font', family="Helvetica Neue")


def good_box(bp, color):
    for box in bp['boxes']:
        box.set(color=color, linewidth=1)
    for w in bp['whiskers']:
        w.set(color=color, linewidth=1)
    for f in bp['fliers']:
        f.set(mec="gray", alpha=0.5, linewidth=1, ms=1)
    for m in bp['medians']:
        m.set(linewidth=2, color="C3")
    for c in bp['caps']:
        c.set(linewidth=1, color='k')


def hard_sigmoid(x, p1, p2, p3):
    return p1 * (x - p2) * (x >= p2) * (p1 * (x - p2) < p3) + p3 * (p1 * (x - p2) >= p3)


def hard_sigmoid_with_noiselevel(x, p1, p2, p3, const):
    return np.sqrt(const ** 2 + hard_sigmoid(x, p1, p2, p3) ** 2)


# initialize the figure and a subplot
fig_w = plt.figure(figsize=(18, 18))
ax_w2 = fig_w.add_subplot(334)

# load the data
waves = np.loadtxt('auditory_LFP_responses_2kHz.txt')

# plot the waves
factor = 0.1
cmap1 = pylustrator.LabColormap(["c", "b"], len(waves[:, 0]))
color = cmap1(len(waves[:, 0])//2)
for i in range(len(waves[:, 0])):
    ax_w2.plot(np.arange(len(waves[0, 1::])), waves[i, 1::] + factor * i, linestyle='-', color=cmap1(i))
# add a line to illustrate the stimulus end
ax_w2.plot([200, 200], [-1, factor * (i + 0.5)], 'k--', lw=0.8)

# add ticks and labels
ax_w2.set_yticks(np.arange(0, i * factor + factor, factor))
loudness = np.arange(30, 95, 5)
ax_w2.set_yticklabels(map(str, loudness))
ax_w2.set_xlabel('time (ms)')
ax_w2.set_ylabel('sound level\n(dB SPL)')
ax_w2.set_ylim(0 - factor / 2, i * factor + factor)

# add a scale bar
y_off = 0.5*factor
x = 420.0
length = 100
plt.plot([x, x], [y_off, y_off+length*1e-3], clip_on=False, color="k")

# Plot b, RMS data
# load the data
rms = np.loadtxt('auditory_LFP_RMS_amplitudes.txt')
params = np.loadtxt('auditory_LFP_RMS_amplitudes_parameters.txt')

# add the subplot
ax2 = fig_w.add_subplot(335)

# plot the fitted line
ax2.plot(np.arange(0, 100, 0.1),
         hard_sigmoid_with_noiselevel(np.arange(0, 100, 0.1), params[0], params[1], params[2], params[3]) * 1000,
         color=color, linewidth=1, alpha=1, clip_on=False)

# plot the baselevel
ax2.plot([0, 90], np.array([params[3], params[3]]) * 1000, linestyle='--', color="k", lw=0.8)#"#color=colorx2)

# set labels and ticks
ax2.set_ylabel('RMS ($\mu$V)', labelpad=-2)
ax2.set_xlabel('sound level (dB SPL)')
ax2.set_xlim(0, 100)
ax2.set_ylim(0, 20)
ax2.plot(rms[:, 0], rms[:, 1] * 1000, marker='s', color=color, linewidth=0, clip_on=False, ms=3)

# Plot c, thresholds
ax02 = fig_w.add_subplot(336)

# load the data
trial_sample = np.arange(30, 300, 10)
trial_number_threshs = np.loadtxt('auditory_LFP_thresholds.txt')

# add the box plot
bp = ax02.boxplot(np.transpose(trial_number_threshs), positions=trial_sample, whis=[5, 95], widths=4)
good_box(bp, color)

# set ticks and labels
ax02.set_xlim(50, max(trial_sample) + 10)
ax02.set_xticks(trial_sample[2::5])
ax02.set_xticks(trial_sample, minor=True)
ax02.minorticks_on()
ax02.set_xticklabels(trial_sample[2::5])
ax02.set_xlabel('sample size')
ax02.set_ylabel('threshold\n(dB SPL)')
ax02.set_ylim(10, 60)


# Somatosensory data

# Plot d, waves
ax_w2s = fig_w.add_subplot(337)

# load the data
waves = np.loadtxt('somatosensory_LFP_responses_175Hz.txt')

# plot the waves
factor = 0.01
cmap1 = pylustrator.LabColormap(["c", "b"], len(waves[:, 0]))
color = cmap1(len(waves[:, 0])//2)
for i in range(len(waves[:, 0])):
    ax_w2s.plot(np.arange(len(waves[0, 1::])), waves[i, 1::] + factor * i, linestyle='-', color=cmap1(i))

# plot a line to illustrate the stimulus end
ax_w2s.plot([200, 200], [-1, factor * (i + 0.5)], 'k--', lw=0.8)

# set the ticks and labels
ax_w2s.set_yticks(np.arange(0, i * factor + factor, factor))
loudness = np.arange(6, 13, 1)
ax_w2s.set_yticklabels(map(str, loudness))
ax_w2s.set_xlabel('time (ms)')
ax_w2s.set_ylabel('deflection\n(dB ref: 1$\mu$m)')
ax_w2s.set_ylim(0 - factor / 2, i * factor + factor)

# add a scale bar
y_off = 0.5*factor
x = 420.0
length = 10
plt.plot([x, x], [y_off, y_off+length*1e-3], clip_on=False, color="k")

# Plot e, the RMS data
ax2s = fig_w.add_subplot(338)

# load the data
rms = np.loadtxt('somatosensory_LFP_RMS_amplitudes.txt')
params = np.loadtxt('somatosensory_LFP_RMS_amplitudes_parameters.txt')

# plot the fitted line
ax2s.plot(np.arange(0, 100, 0.1),
          hard_sigmoid_with_noiselevel(np.arange(0, 100, 0.1), params[0], params[1], params[2], params[3]) * 1000, color=color,
          linewidth=1, alpha=1)

# plot the base level
ax2s.plot([0, 90], np.array([params[3], params[3]]) * 1000, linestyle='--', color="k", lw=0.8)#color=colorx2)

# plot the data points
ax2s.plot(rms[:, 0], rms[:, 1] * 1000, marker='s', color=color, linewidth=0, clip_on=False, ms=3)

# set the label and ticks
ax2s.set_ylabel('RMS ($\mu$V)')
ax2s.set_xlabel('deflection (dB ref: 1$\mu$m)')
ax2s.set_xlim(0, 15)
ax2s.set_ylim(0, 2.7)
ax2s.set_xticks(np.arange(0.0, 15, 4))

# Plot f, thresholds
ax02s = fig_w.add_subplot(339)

# load the data
trial_sample = np.arange(30, 300, 10)
trial_number_threshs = np.loadtxt('somatosensory_LFP_thresholds.txt')

# plot the boxes
bp = ax02s.boxplot(np.transpose(trial_number_threshs), positions=trial_sample, whis=[5, 95], widths=4)
good_box(bp, color)

# ticks and labels
ax02s.set_xlim(50, max(trial_sample) + 10)
ax02s.set_xticks(trial_sample[2::5])
ax02s.set_xticks(trial_sample, minor=True)
ax02s.minorticks_on()
ax02s.set_xticklabels(trial_sample[2::5])
ax02s.set_xlabel('sample size')
ax02s.set_ylabel('threshold\n(dB ref: 1$\mu$m)')
ax02s.set_ylim(-4, 8)

#% start: automatic generated code from pylustrator
fig = plt.figure(1)
import matplotlib as mpl
fig.ax_dict = {ax.get_label(): ax for ax in fig.axes}
fig.set_size_inches(15.980000/2.54, 8.980000/2.54, forward=True)
fig.axes[0].text(0.5, 0.5, 'New Text', transform=fig.axes[0].transAxes)
fig.axes[0].text(0.5, 0.5, 'New Text', transform=fig.axes[0].transAxes, rotation=-90)
fig.axes[0].set_position([0.104132, 0.651271, 0.227505, 0.284648])
fig.axes[0].spines['right'].set_visible(False)
fig.axes[0].spines['top'].set_visible(False)
fig.axes[0].texts[0].set_text("auditory")
fig.axes[0].texts[0].set_ha("right")
fig.axes[0].texts[0].set_color("#808080")
fig.axes[0].texts[0].set_position([1.002180, 0.931360])  # Text: "Auditory"
fig.axes[0].texts[1].set_ha("center")
fig.axes[0].texts[1].set_fontsize(8)
fig.axes[0].texts[1].set_text(u"100 µV")
fig.axes[0].texts[1].set_position([1.002180, 0.300957])  # Text: "100 �V"
fig.axes[0].text(0.5, 0.5, 'New Text', transform=fig.axes[0].transAxes)  # id=fig.axes[0].texts[2].new
fig.axes[0].texts[2].set_text("a")
fig.axes[0].texts[2].set_weight("bold")
fig.axes[0].texts[2].set_position([-0.275678, 1.037416])
fig.axes[1].set_position([0.437470, 0.651271, 0.210153, 0.284648])
fig.axes[1].spines['right'].set_visible(False)
fig.axes[1].spines['top'].set_visible(False)
fig.axes[1].text(0.5, 0.5, 'New Text', transform=fig.axes[1].transAxes)  # id=fig.axes[1].texts[0].new
fig.axes[1].texts[0].set_text("b")
fig.axes[1].texts[0].set_weight("bold")
fig.axes[1].texts[0].set_position([-0.301899, 1.037416])
fig.axes[2].text(0.5, 0.5, 'New Text', transform=fig.axes[2].transAxes)
fig.axes[2].text(0.5, 0.5, 'New Text', transform=fig.axes[2].transAxes, rotation=-90)
fig.axes[2].set_position([0.753456, 0.651271, 0.227505, 0.284648])
fig.axes[2].spines['right'].set_visible(False)
fig.axes[2].spines['top'].set_visible(False)
fig.axes[2].yaxis.labelpad = 1.760000
fig.axes[2].set_zorder(1)
fig.axes[2].texts[0].set_visible(False)
fig.axes[2].texts[1].set_text(u"10 µV")
fig.axes[2].texts[1].set_fontsize(8)
fig.axes[2].texts[1].set_ha("center")
fig.axes[2].texts[1].set_position([-1.851929, -1.410813])
fig.axes[2].text(0.5, 0.5, 'New Text', transform=fig.axes[2].transAxes)  # id=fig.axes[2].texts[2].new
fig.axes[2].texts[2].set_text("c")
fig.axes[2].texts[2].set_weight("bold")
fig.axes[2].texts[2].set_position([-0.316040, 1.037416])
fig.axes[3].set_position([0.104132, 0.178363, 0.227505, 0.284327])
fig.axes[3].spines['right'].set_visible(False)
fig.axes[3].spines['top'].set_visible(False)
fig.axes[3].text(0.5, 0.5, 'New Text', transform=fig.axes[3].transAxes)  # id=fig.axes[3].texts[0].new
fig.axes[3].texts[0].set_text("d")
fig.axes[3].texts[0].set_weight("bold")
fig.axes[3].texts[0].set_position([-0.275678, 0.980703])
fig.axes[3].text(0.5, 0.5, 'New Text', transform=fig.axes[3].transAxes)  # id=fig.axes[3].texts[1].new
fig.axes[3].texts[1].set_text("somatosensory")
fig.axes[3].texts[1].set_color("#808080ff")
fig.axes[3].texts[1].set_position([0.374214, 0.980703])
fig.axes[4].set_position([0.437470, 0.178042, 0.210153, 0.284648])
fig.axes[4].spines['right'].set_visible(False)
fig.axes[4].spines['top'].set_visible(False)
fig.axes[4].text(0.5, 0.5, 'New Text', transform=fig.axes[4].transAxes)  # id=fig.axes[4].texts[0].new
fig.axes[4].texts[0].set_text("e")
fig.axes[4].texts[0].set_weight("bold")
fig.axes[4].texts[0].set_position([-0.301900, 0.980725])
fig.axes[5].set_position([0.753456, 0.178042, 0.227505, 0.284648])
fig.axes[5].spines['right'].set_visible(False)
fig.axes[5].spines['top'].set_visible(False)
fig.axes[5].yaxis.labelpad = 2.400000
fig.axes[5].text(0.5, 0.5, 'New Text', transform=fig.axes[5].transAxes)  # id=fig.axes[5].texts[0].new
fig.axes[5].texts[0].set_text("f")
fig.axes[5].texts[0].set_weight("bold")
fig.axes[5].texts[0].set_position([-0.316042, 0.980725])
#% end: automatic generated code from pylustrator

plt.savefig(__file__[:-3]+".pdf", dpi=300)
plt.savefig(__file__[:-3]+".png", dpi=300)

plt.show()
